@@ -1,4 +1,4 @@
-use chrono::{DateTime, NaiveDateTime, Utc};
+use chrono::{DateTime, Utc};
 use std::env;
 use std::process::Command;
 
@@ -8,11 +8,7 @@ fn main() {
 
     // get timestamp
     let now = match env::var("SOURCE_DATE_EPOCH") {
-        Ok(val) => {
-            let naive = NaiveDateTime::from_timestamp_opt(val.parse::<i64>().unwrap(), 0).unwrap();
-            let datetime: DateTime<Utc> = DateTime::from_utc(naive, Utc);
-            datetime
-        }
+        Ok(val) => DateTime::from_timestamp(val.parse::<i64>().unwrap(), 0).unwrap(),
         Err(_) => Utc::now(),
     };
     println!("cargo:rustc-env=BUILD_TIMESTAMP={}", now.to_rfc3339());
@@ -30,5 +26,5 @@ fn main() {
         // if error, e.g. build from source with git repo, just show empty string
         Err(_) => "".to_string(),
     };
-    println!("cargo:rustc-env=GIT_COMMIT={}", commit);
+    println!("cargo:rustc-env=GIT_COMMIT={commit}");
 }
