@@ -5,37 +5,61 @@
 // following tests will not test server and event loop since
 // event-loop and server can be tested via integration tests
 mod tests {
+    use std::collections::HashMap;
     use std::net::{IpAddr, Ipv4Addr, Ipv6Addr};
 
-    use aardvark_dns::backend::DNSResult;
+    use aardvark_dns::backend::DNSBackend;
     use aardvark_dns::config;
+    use aardvark_dns::error::AardvarkResult;
     use std::str::FromStr;
+
+    const IP_10_88_0_2: IpAddr = IpAddr::V4(Ipv4Addr::new(10, 88, 0, 2));
+    const IP_10_88_0_4: IpAddr = IpAddr::V4(Ipv4Addr::new(10, 88, 0, 4));
+    const IP_10_88_0_5: IpAddr = IpAddr::V4(Ipv4Addr::new(10, 88, 0, 5));
+
+    const IP_10_89_0_2: IpAddr = IpAddr::V4(Ipv4Addr::new(10, 89, 0, 2));
+    const IP_10_89_0_3: IpAddr = IpAddr::V4(Ipv4Addr::new(10, 89, 0, 3));
+
+    /// fdfd:733b:dc3:220b::2
+    const IP_FDFD_733B_DC3_220B_2: IpAddr =
+        IpAddr::V6(Ipv6Addr::new(0xfdfd, 0x733b, 0xdc3, 0x220b, 0, 0, 0, 2));
+    /// fdfd:733b:dc3:220b::3
+    const IP_FDFD_733B_DC3_220B_3: IpAddr =
+        IpAddr::V6(Ipv6Addr::new(0xfdfd, 0x733b, 0xdc3, 0x220b, 0, 0, 0, 3));
+
+    fn parse_configs(
+        dir: &str,
+    ) -> AardvarkResult<(
+        DNSBackend,
+        HashMap<String, Vec<Ipv4Addr>>,
+        HashMap<String, Vec<Ipv6Addr>>,
+    )> {
+        config::parse_configs(dir, "")
+    }
+
     /* -------------------------------------------- */
     // --------- Test aardvark-dns config ---------
     /* -------------------------------------------- */
     #[test]
     // Test loading of config file from directory
     fn test_loading_config_file() {
-        match config::parse_configs("src/test/config/podman") {
-            Ok(_) => {}
-            Err(e) => panic!("{}", e),
-        }
+        parse_configs("src/test/config/podman").unwrap();
     }
     #[test]
     // Test loading of config file from directory with custom DNS for containers
     fn test_loading_config_file_with_dns_servers() {
-        config::parse_configs("src/test/config/podman_custom_dns_servers").unwrap();
+        parse_configs("src/test/config/podman_custom_dns_servers").unwrap();
     }
     #[test]
     // Test loading of config file from directory with custom DNS for containers
     // and custom DNS servers at network level as well.
     fn test_loading_config_file_with_network_scoped_dns_servers() {
-        config::parse_configs("src/test/config/network_scoped_custom_dns").unwrap();
+        parse_configs("src/test/config/network_scoped_custom_dns").unwrap();
     }
     #[test]
     // Parse config files from stub data
     fn test_parsing_config_files() {
-        match config::parse_configs("src/test/config/podman") {
+        match parse_configs("src/test/config/podman") {
             Ok((_, listen_ip_v4, _)) => {
                 listen_ip_v4.contains_key("podman");
                 assert_eq!(listen_ip_v4["podman"].len(), 1);
@@ -45,12 +69,9 @@ mod tests {
         }
     }
     #[test]
-    // Parse bad config files must fail
+    // Parse bad config files should not hard error
     fn test_parsing_bad_config_files() {
-        match config::parse_configs("src/test/config/podman_bad_config") {
-            Ok((_, _, _)) => panic!("parsing bad config must fail"),
-            Err(_) => {}
-        }
+        parse_configs("src/test/config/podman_bad_config").expect("config parsing failed");
     }
     /* -------------------------------------------- */
     // -------Verify backend custom dns server ----
@@ -59,7 +80,7 @@ mod tests {
     // Backend must populate ctr_dns_servers via custom
     // DNS servers for container from the aardvark config
     fn test_backend_custom_dns_server() {
-        match config::parse_configs("src/test/config/podman_custom_dns_servers") {
+        match parse_configs("src/test/config/podman_custom_dns_servers") {
             Ok((backend, _, _)) => {
                 // Should contain custom DNS server 8.8.8.8
                 let mut dns_server = backend
@@ -94,10 +115,13 @@ mod tests {
     // DNS servers for container from container entry and
     // network dns servers as well.
     fn test_backend_network_scoped_custom_dns_server() {
-        match config::parse_configs("src/test/config/network_scoped_custom_dns") {
+        match parse_configs("src/test/config/network_scoped_custom_dns") {
             Ok((backend, _, _)) => {
-                let expected_dnsservers = vec!["127.0.0.1", "::0.0.0.2"];
-                let test_cases_source = vec!["10.88.0.2", "10.88.0.3", "10.88.0.4", "10.88.0.5"];
+                let expected_dnsservers = vec![
+                    IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)),
+                    IpAddr::V6(Ipv6Addr::new(0, 0, 0, 0, 0, 0, 0, 2)),
+                ];
+                let test_cases_source = ["10.88.0.2", "10.88.0.3", "10.88.0.4", "10.88.0.5"];
                 // verify if network scoped resolvers for all the containers is equivalent to
                 // expectedDNSServers
                 for container in test_cases_source.iter() {
@@ -105,7 +129,7 @@ mod tests {
                         backend.get_network_scoped_resolvers(&IpAddr::from_str(container).unwrap());
                     let mut output_dnsservers = Vec::new();
                     for server in output.unwrap().iter() {
-                        output_dnsservers.push(format!("{}", server));
+                        output_dnsservers.push(*server);
                     }
                     assert_eq!(expected_dnsservers, output_dnsservers);
                 }
@@ -124,18 +148,11 @@ mod tests {
     // Request address must be v4.
     // Same container --> (resolve) Same container name --> (on) Same Network
     fn test_lookup_queries_from_backend_simulate_same_container_request_from_v4_on_v4_entries() {
-        match config::parse_configs("src/test/config/podman") {
-            Ok((backend, _, _)) => {
-                match backend.lookup(&"10.88.0.2".parse().unwrap(), "condescendingnash") {
-                    DNSResult::Success(ip_vec) => {
-                        assert_eq!(ip_vec.len(), 1);
-                        assert_eq!("10.88.0.2".parse(), Ok(ip_vec[0]));
-                    }
-                    _ => panic!("unexpected dns result"),
-                }
-            }
-            Err(e) => panic!("{}", e),
-        }
+        let res = parse_configs("src/test/config/podman")
+            .expect("parse config error")
+            .0
+            .lookup(&IP_10_88_0_2, "", "condescendingnash");
+        assert_eq!(res, Some(vec![IP_10_88_0_2]));
     }
     #[test]
     // Check lookup query from backend and simulate
@@ -145,18 +162,11 @@ mod tests {
     // Same container --> (resolve) Same container name --> (on) Same Network
     fn test_lookup_queries_from_backend_simulate_same_container_request_from_v4_on_v4_entries_case_insensitive(
     ) {
-        match config::parse_configs("src/test/config/podman") {
-            Ok((backend, _, _)) => {
-                match backend.lookup(&"10.88.0.2".parse().unwrap(), "helloworld") {
-                    DNSResult::Success(ip_vec) => {
-                        assert_eq!(ip_vec.len(), 1);
-                        assert_eq!("10.88.0.5".parse(), Ok(ip_vec[0]));
-                    }
-                    _ => panic!("unexpected dns result"),
-                }
-            }
-            Err(e) => panic!("{}", e),
-        }
+        let res = parse_configs("src/test/config/podman")
+            .expect("parse config error")
+            .0
+            .lookup(&IP_10_88_0_2, "", "helloworld");
+        assert_eq!(res, Some(vec![IP_10_88_0_5]));
     }
     #[test]
     // Check lookup query from backend and simulate
@@ -166,32 +176,21 @@ mod tests {
     // Same container --> (resolve) Same container name --> (on) Same Network
     fn test_lookup_queries_from_backend_simulate_same_container_request_from_v4_on_v4_entries_case_insensitive_uppercase(
     ) {
-        match config::parse_configs("src/test/config/podman") {
-            Ok((backend, _, _)) => {
-                match backend.lookup(&"10.88.0.2".parse().unwrap(), "HELLOWORLD") {
-                    DNSResult::Success(ip_vec) => {
-                        assert_eq!(ip_vec.len(), 1);
-                        assert_eq!("10.88.0.5".parse(), Ok(ip_vec[0]));
-                    }
-                    _ => panic!("unexpected dns result"),
-                }
-            }
-            Err(e) => panic!("{}", e),
-        }
+        let res = parse_configs("src/test/config/podman")
+            .expect("parse config error")
+            .0
+            .lookup(&IP_10_88_0_2, "", "HELLOWORLD");
+        assert_eq!(res, Some(vec![IP_10_88_0_5]));
     }
     #[test]
     // Check lookup query from backend and simulate
     // nx_domain on bad lookup queries.
     fn test_lookup_queries_from_backend_simulate_nx_domain() {
-        match config::parse_configs("src/test/config/podman") {
-            Ok((backend, _, _)) => {
-                match backend.lookup(&"10.88.0.2".parse().unwrap(), "somebadquery") {
-                    DNSResult::NXDomain => {}
-                    _ => panic!("unexpected dns result"),
-                }
-            }
-            Err(e) => panic!("{}", e),
-        }
+        let res = parse_configs("src/test/config/podman")
+            .expect("parse config error")
+            .0
+            .lookup(&IP_10_88_0_2, "", "somebadquery");
+        assert_eq!(res, None);
     }
     #[test]
     // Check lookup query from backend and simulate
@@ -200,18 +199,11 @@ mod tests {
     // Request address must be v4.
     // Same container --> (resolve) different container name --> (on) Same Network
     fn test_lookup_queries_from_backend_simulate_different_container_request_from_v4() {
-        match config::parse_configs("src/test/config/podman") {
-            Ok((backend, _, _)) => {
-                match backend.lookup(&"10.88.0.2".parse().unwrap(), "trustingzhukovsky") {
-                    DNSResult::Success(ip_vec) => {
-                        assert_eq!(ip_vec.len(), 1);
-                        assert_eq!("10.88.0.4".parse(), Ok(ip_vec[0]));
-                    }
-                    _ => panic!("unexpected dns result"),
-                }
-            }
-            Err(e) => panic!("{}", e),
-        }
+        let res = parse_configs("src/test/config/podman")
+            .expect("parse config error")
+            .0
+            .lookup(&IP_10_88_0_2, "", "trustingzhukovsky");
+        assert_eq!(res, Some(vec![IP_10_88_0_4]));
     }
     #[test]
     // Check lookup query from backend and simulate
@@ -220,117 +212,42 @@ mod tests {
     // Request address must be v4.
     // Same container --> (resolve) different container name by alias --> (on) Same Network
     fn test_lookup_queries_from_backend_simulate_different_container_request_from_v4_by_alias() {
-        match config::parse_configs("src/test/config/podman") {
-            Ok((backend, _, _)) => match backend.lookup(&"10.88.0.2".parse().unwrap(), "ctr1") {
-                DNSResult::Success(ip_vec) => {
-                    // verfiy length for issues like: https://github.com/containers/aardvark-dns/issues/120
-                    assert_eq!(ip_vec.len(), 1);
-                    assert_eq!("10.88.0.4".parse(), Ok(ip_vec[0]));
-                }
-                _ => panic!("unexpected dns result"),
-            },
-            Err(e) => panic!("{}", e),
-        }
+        let res = parse_configs("src/test/config/podman")
+            .expect("parse config error")
+            .0
+            .lookup(&IP_10_88_0_2, "", "ctr1");
+        assert_eq!(res, Some(vec![IP_10_88_0_4]));
     }
     #[test]
     // Check lookup query from backend and simulate
     // dns request from same container to itself but
     // aardvark must return two ip address for v4 and v6.
-    // Request address must be v4.
     // Same container --> (resolve) Same container name --> (on) Same Network
-    fn test_lookup_queries_from_backend_simulate_same_container_request_from_v4_on_v6_and_v4_entries(
-    ) {
-        match config::parse_configs("src/test/config/podman_v6_entries") {
-            Ok((backend, listen_ip_v4, listen_ip_v6)) => {
-                listen_ip_v6.contains_key("podman_v6_entries");
-                listen_ip_v4.contains_key("podman_v6_entries");
-                match backend.lookup(&"10.89.0.2".parse().unwrap(), "test1") {
-                    DNSResult::Success(ip_vec) => {
-                        // verfiy length for issues like: https://github.com/containers/aardvark-dns/issues/120
-                        assert_eq!(ip_vec.len(), 2);
-                        assert_eq!("10.89.0.2".parse(), Ok(ip_vec[0]));
-                        assert_eq!("fdfd:733b:dc3:220b::2".parse(), Ok(ip_vec[1]));
-                    }
-                    _ => panic!("unexpected dns result"),
-                }
-            }
-            Err(e) => panic!("{}", e),
-        }
-    }
-    #[test]
-    // Check lookup query from backend and simulate
-    // dns request from same container to itself but
-    // aardvark must return two ip address for v4 and v6.
-    // Request address must be v6.
-    // Same container --> (resolve) Same container name --> (on) Same Network
-    fn test_lookup_queries_from_backend_simulate_same_container_request_from_v6_on_v6_and_v4_entries(
-    ) {
-        match config::parse_configs("src/test/config/podman_v6_entries") {
-            Ok((backend, listen_ip_v4, listen_ip_v6)) => {
-                listen_ip_v6.contains_key("podman_v6_entries");
-                listen_ip_v4.contains_key("podman_v6_entries");
-                match backend.lookup(&"fdfd:733b:dc3:220b::2".parse().unwrap(), "test1") {
-                    DNSResult::Success(ip_vec) => {
-                        // verfiy length for issues like: https://github.com/containers/aardvark-dns/issues/120
-                        assert_eq!(ip_vec.len(), 2);
-                        assert_eq!("10.89.0.2".parse(), Ok(ip_vec[0]));
-                        assert_eq!("fdfd:733b:dc3:220b::2".parse(), Ok(ip_vec[1]));
-                    }
-                    _ => panic!("unexpected dns result"),
-                }
-            }
-            Err(e) => panic!("{}", e),
-        }
+    fn test_lookup_queries_from_backend_simulate_same_container_request_from_v4_and_v6_entries() {
+        let conf = parse_configs("src/test/config/podman_v6_entries").expect("parse config error");
+        assert!(conf.1.contains_key("podman_v6_entries"));
+        assert!(!conf.2.contains_key("podman_v6_entries"));
+
+        let ips = conf.0.lookup(&IP_10_89_0_2, "", "test1");
+        assert_eq!(ips, Some(vec![IP_10_89_0_2, IP_FDFD_733B_DC3_220B_2]));
+        let ips = conf.0.lookup(&IP_FDFD_733B_DC3_220B_2, "", "test1");
+        assert_eq!(ips, Some(vec![IP_10_89_0_2, IP_FDFD_733B_DC3_220B_2]));
     }
     #[test]
     // Check lookup query from backend and simulate
     // dns request from container to another container but
     // aardvark must return two ip address for v4 and v6.
-    // Request address must be v6.
     // Same container --> (resolve) different container name --> (on) Same Network
-    fn test_lookup_queries_from_backend_simulate_different_container_request_from_v6_on_v6_and_v4_entries(
+    fn test_lookup_queries_from_backend_simulate_different_container_request_from_v4_and_v6_entries(
     ) {
-        match config::parse_configs("src/test/config/podman_v6_entries") {
-            Ok((backend, listen_ip_v4, listen_ip_v6)) => {
-                listen_ip_v6.contains_key("podman_v6_entries");
-                listen_ip_v4.contains_key("podman_v6_entries");
-                match backend.lookup(&"fdfd:733b:dc3:220b::2".parse().unwrap(), "test2") {
-                    DNSResult::Success(ip_vec) => {
-                        // verfiy length for issues like: https://github.com/containers/aardvark-dns/issues/120
-                        assert_eq!(ip_vec.len(), 2);
-                        assert_eq!("10.89.0.3".parse(), Ok(ip_vec[0]));
-                        assert_eq!("fdfd:733b:dc3:220b::3".parse(), Ok(ip_vec[1]));
-                    }
-                    _ => panic!("unexpected dns result"),
-                }
-            }
-            Err(e) => panic!("{}", e),
-        }
-    }
-    #[test]
-    // Check lookup query from backend and simulate
-    // dns request from container to another container but
-    // aardvark must return two ip address for v4 and v6.
-    // Request address must be v6.
-    // Same container --> (resolve) different container name --> (on) Same Network
-    fn test_lookup_queries_from_backend_simulate_different_container_request_from_v4_on_v6_and_v4_entries(
-    ) {
-        match config::parse_configs("src/test/config/podman_v6_entries") {
-            Ok((backend, listen_ip_v4, listen_ip_v6)) => {
-                listen_ip_v6.contains_key("podman_v6_entries");
-                listen_ip_v4.contains_key("podman_v6_entries");
-                match backend.lookup(&"10.89.0.2".parse().unwrap(), "test2") {
-                    DNSResult::Success(ip_vec) => {
-                        // verfiy length for issues like: https://github.com/containers/aardvark-dns/issues/120
-                        assert_eq!(ip_vec.len(), 2);
-                        assert_eq!("10.89.0.3".parse(), Ok(ip_vec[0]));
-                        assert_eq!("fdfd:733b:dc3:220b::3".parse(), Ok(ip_vec[1]));
-                    }
-                    _ => panic!("unexpected dns result"),
-                }
-            }
-            Err(e) => panic!("{}", e),
-        }
+        let conf = parse_configs("src/test/config/podman_v6_entries").expect("parse config error");
+        assert!(conf.1.contains_key("podman_v6_entries"));
+        assert!(!conf.2.contains_key("podman_v6_entries"));
+
+        let ips = conf.0.lookup(&IP_10_89_0_2, "", "test2");
+        assert_eq!(ips, Some(vec![IP_10_89_0_3, IP_FDFD_733B_DC3_220B_3]));
+        let ips = conf.0.lookup(&IP_FDFD_733B_DC3_220B_2, "", "test2");
+        assert_eq!(ips, Some(vec![IP_10_89_0_3, IP_FDFD_733B_DC3_220B_3]));
     }
     #[test]
     // Check lookup query from backend and simulate
@@ -340,22 +257,12 @@ mod tests {
     // Same container --> (resolve) different container by id --> (on) Same Network
     fn test_lookup_queries_from_backend_simulate_different_container_request_by_id_from_v4_on_v6_and_v4_entries(
     ) {
-        match config::parse_configs("src/test/config/podman_v6_entries") {
-            Ok((backend, listen_ip_v4, listen_ip_v6)) => {
-                listen_ip_v6.contains_key("podman_v6_entries");
-                listen_ip_v4.contains_key("podman_v6_entries");
-                match backend.lookup(&"10.89.0.2".parse().unwrap(), "88dde8a24897") {
-                    DNSResult::Success(ip_vec) => {
-                        // verfiy length for issues like: https://github.com/containers/aardvark-dns/issues/120
-                        assert_eq!(ip_vec.len(), 2);
-                        assert_eq!("10.89.0.3".parse(), Ok(ip_vec[0]));
-                        assert_eq!("fdfd:733b:dc3:220b::3".parse(), Ok(ip_vec[1]));
-                    }
-                    _ => panic!("unexpected dns result"),
-                }
-            }
-            Err(e) => panic!("{}", e),
-        }
+        let conf = parse_configs("src/test/config/podman_v6_entries").expect("parse config error");
+        assert!(conf.1.contains_key("podman_v6_entries"));
+        assert!(!conf.2.contains_key("podman_v6_entries"));
+
+        let ips = conf.0.lookup(&IP_10_89_0_2, "", "88dde8a24897");
+        assert_eq!(ips, Some(vec![IP_10_89_0_3, IP_FDFD_733B_DC3_220B_3]));
     }
     /* -------------------------------------------- */
     // ---Test aardvark-dns reverse lookup logic --
@@ -366,7 +273,7 @@ mod tests {
     // aardvark must return container name and alias
     // Same container --> (resolve) Same ip  --> (on) Same Network
     fn test_reverse_lookup_queries_from_backend_by_ip_v4() {
-        match config::parse_configs("src/test/config/podman") {
+        match parse_configs("src/test/config/podman") {
             Ok((backend, _, _)) => {
                 match backend
                     .reverse_lookup(&"10.88.0.4".parse().unwrap(), &"10.88.0.4".parse().unwrap())
@@ -393,7 +300,7 @@ mod tests {
     // aardvark must return container name and alias
     // Same container --> (resolve) Same ip  --> (on) Same Network
     fn test_reverse_lookup_queries_from_backend_by_ip_v6() {
-        match config::parse_configs("src/test/config/podman_v6_entries") {
+        match parse_configs("src/test/config/podman_v6_entries") {
             Ok((backend, _, _)) => {
                 match backend.reverse_lookup(
                     &"fdfd:733b:dc3:220b::2".parse().unwrap(),
@@ -417,7 +324,7 @@ mod tests {
     #[test]
     // Check ip_mappings generated by backend
     fn test_generated_ip_mappings_in_backend() {
-        match config::parse_configs("src/test/config/podman_v6_entries") {
+        match parse_configs("src/test/config/podman_v6_entries") {
             Ok((backend, listen_ip_v4, listen_ip_v6)) => {
                 listen_ip_v6.contains_key("podman_v6_entries");
                 listen_ip_v4.contains_key("podman_v6_entries");
@@ -442,7 +349,7 @@ mod tests {
     #[test]
     // Check name_mappings generated by backend
     fn test_generated_name_mappings_in_backend() {
-        match config::parse_configs("src/test/config/podman_v6_entries") {
+        match parse_configs("src/test/config/podman_v6_entries") {
             Ok((backend, listen_ip_v4, listen_ip_v6)) => {
                 listen_ip_v6.contains_key("podman_v6_entries");
                 listen_ip_v4.contains_key("podman_v6_entries");
@@ -496,7 +403,7 @@ mod tests {
     #[test]
     // Check reverse_mappings generated by backend
     fn test_generated_reverse_mappings_in_backend() {
-        match config::parse_configs("src/test/config/podman_v6_entries") {
+        match parse_configs("src/test/config/podman_v6_entries") {
             Ok((backend, listen_ip_v4, listen_ip_v6)) => {
                 listen_ip_v6.contains_key("podman_v6_entries");
                 listen_ip_v4.contains_key("podman_v6_entries");
@@ -517,7 +424,7 @@ mod tests {
     #[test]
     // Parse a config which contains multiple ipv4 and ipv6 addresses ona single line
     fn test_parse_multiple_ipv4_ipv6_addresses() {
-        match config::parse_configs("src/test/config/podman_v6_entries") {
+        match parse_configs("src/test/config/podman_v6_entries") {
             Ok((backend, listen_ip_v4, listen_ip_v6)) => {
                 assert_eq!(
                     listen_ip_v4["podman_v6_entries_proper"],
@@ -533,8 +440,8 @@ mod tests {
                         "fddd::1".parse().unwrap()
                     ]
                 );
-                match backend.lookup(&"10.0.0.2".parse().unwrap(), "testmulti1") {
-                    DNSResult::Success(ip_vec) => {
+                match backend.lookup(&"10.0.0.2".parse().unwrap(), "", "testmulti1") {
+                    Some(ip_vec) => {
                         assert_eq!(
                             ip_vec,
                             vec![
@@ -548,8 +455,8 @@ mod tests {
                     _ => panic!("unexpected dns result"),
                 }
 
-                match backend.lookup(&"10.0.0.2".parse().unwrap(), "testmulti2") {
-                    DNSResult::Success(ip_vec) => {
+                match backend.lookup(&"10.0.0.2".parse().unwrap(), "", "testmulti2") {
+                    Some(ip_vec) => {
                         assert_eq!(
                             ip_vec,
                             vec![
